#include "firepro_viewer.h"
#include "box.h"
#include "header_cont.h"
#include "header_disc.h"
#include "seqlogo.h"
#include "box_text.h"
#include "box_bin.h"
#include "parser.h"
#include "scalebar.h"

#include <QtGui>
#include <QGraphicsSvgItem>
#include <QDebug>

#include <iostream>
#include <string>
#include <vector>
#include <sstream>
#include <algorithm>
#include <iterator>
#include <math.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <fstream>
#include <map>
#include <algorithm>

using namespace std;

firepro_viewer::firepro_viewer(string filename)
{
    firepro_results results = parse_firepro(filename);

    if (results.general.status == 1 && results.general.motifs.size() > 0) {
    } else {
        exit(1);
    }

    this->setWindowTitle(tr("FIRE-pro: ") + QString::fromStdString(filename));
    this->setWindowIcon(QIcon(":/icon/icon"));

    int motifcount = results.general.motifs.size();
    int bincount = results.general.bincount;
    int clusters = results.general.clusters;
    string exptype = results.general.exptype;

    // order motifs by mi and cluster
    vector<protein_motif> ordered_candidates;
    for (int i = 0; i < clusters; i++) {
        for (int j = 0; j < motifcount; j++) {
            if (results.general.motifs[j].cluster == i) {
                ordered_candidates.push_back(results.general.motifs[j]);
            }
        }
    }

    // set up stage for drawing
    this->setViewportUpdateMode(FullViewportUpdate);
    this->setRenderHints(QPainter::Antialiasing|QPainter::TextAntialiasing);

    // set up pixel values
    int top_margin = 8;
    int bottom_margin = 8;
    int left_margin = 8;
    int right_margin = 8;

    int y_colorbar_offset = 30;
    int y_offset = 0;
    int y_header_offset = 100;
    int bin_height = 50;

    int x_offset = 0;
    int x_labelwidth = 30;

    int bin_width = 8;
    int cluster_gap = 4;
    int horiz_gap = 12;

    int scalebar_width = 550;

    // determine if space should be allocated for continuous labels
    if (QString::fromStdString(exptype) == "discrete") {
        x_labelwidth = 0;
    }

    // determine scene height, width
    int height = top_margin + y_colorbar_offset + y_offset + y_header_offset + motifcount*bin_height + motifcount + cluster_gap*clusters + bottom_margin; // second motifcount is for spacing out antialiased drawing
    int width = left_margin + x_offset + x_labelwidth + bincount * bin_width + bincount + horiz_gap+ 120 + horiz_gap + 60 + horiz_gap + 60 + horiz_gap + 60 + horiz_gap + 225 + right_margin;

    // increase the width of bins to suit display size
    QRect screenrect = QApplication::desktop()->screenGeometry();
    if (width < screenrect.width()) {
        int difference = (screenrect.width()-width)/bincount;
        if (difference > 16) {
            bin_width += 16;
            width += bincount *16;
        } else {
            bin_width += difference;
            width += bincount *difference;
        }
    }

    // define scene
    QRectF bounds (0, 0, width, height);
    scene = new QGraphicsScene(bounds, this);

    // define background color; currently light grey
    QColor background = QColor(230, 230, 230);

    // add background
    box *base = new box (width, height, background, background, 0, 0, false);
    scene->addItem(base);

    scene->setItemIndexMethod(QGraphicsScene::NoIndex);
    this->setScene(scene);

    // scalebar section
    QGraphicsItem *scale = new scalebar(scalebar_width, y_colorbar_offset, left_margin, top_margin,-30,0,30,":/heatmap/firepro","under-represented", "over-represented");
    scene->addItem(scale);
    // end scalebar section

    int left_x;
    int top_y;

    // metabin drawing
    QGraphicsItem **headerbins = new QGraphicsItem *[bincount];

    if (exptype == "continuous") {
        double metabinmin = results.general.metabins[0].min;
        double metabinmax = results.general.metabins[bincount-1].max;

        QGraphicsItem **continuous_labelbins = new QGraphicsItem *[2];
        continuous_labelbins[0] = new box_text(x_labelwidth-3, y_header_offset/5, background, background, left_margin, top_margin+y_colorbar_offset, QString::number(metabinmax));
        continuous_labelbins[0]->setToolTip(QString::number(metabinmax));
        scene->addItem(continuous_labelbins[0]);

        continuous_labelbins[1] = new box_text(x_labelwidth-3, y_header_offset/5, background, background, left_margin, top_margin+y_colorbar_offset+y_header_offset-y_header_offset/5, QString::number(metabinmin));
        continuous_labelbins[1]->setToolTip(QString::number(metabinmin));
        scene->addItem(continuous_labelbins[1]);

        left_x = left_margin + x_offset + x_labelwidth;
        top_y = top_margin + y_colorbar_offset + y_offset;
        for (int i = 0; i < bincount; i++) {
            left_x = left_margin + x_offset + x_labelwidth+ i*bin_width+i;
            headerbins[i] = new header_cont(bin_width, y_header_offset, left_x, top_y, metabinmin, metabinmax, results.general.metabins[i].min, results.general.metabins[i].max, results.general.metabins[i]);
            QString range = QString::number(results.general.metabins[i].min)+ ", " + QString::number(results.general.metabins[i].max);
            headerbins[i]->setToolTip(range);
            scene->addItem(headerbins[i]);
        }
    } else {
        left_x = left_margin + x_offset + x_labelwidth;
        top_y = top_margin + y_colorbar_offset + y_offset;
        for (int i = 0; i < bincount; i++) {
            left_x = left_margin + x_offset + x_labelwidth+ i*bin_width+i;
            QString metabintext = QString::number(results.general.metabins[i].bin);
            // need to include all headers
            headerbins[i] = new header_disc(bin_width, y_header_offset, left_x, top_y, metabintext, background, results.general.metabins[i]);
            headerbins[i]->setToolTip(metabintext);
            scene->addItem(headerbins[i]);
        }
    }
    // end metabin drawing

    // label bin drawing
    left_x = left_margin + x_offset + x_labelwidth + bincount*bin_width+bincount + horiz_gap;
    top_y = top_margin + y_colorbar_offset + y_offset;

    QGraphicsItem **labelbins = new QGraphicsItem *[5];

    labelbins[0] = new box_text(120, y_header_offset, background, background, left_x, top_y, "candidate", true);
    scene->addItem(labelbins[0]);

    left_x += 120 + horiz_gap;
    labelbins[2] = new box_text(60, y_header_offset, background, background, left_x, top_y, "MI (bits)", true);
    scene->addItem(labelbins[2]);

    left_x += 60 + horiz_gap;
    labelbins[3] = new box_text(60, y_header_offset, background, background, left_x, top_y, "z-score", true);
    scene->addItem(labelbins[3]);

    left_x += 60 + horiz_gap;
    labelbins[4] = new box_text(60, y_header_offset, background, background, left_x, top_y, "robustness", true);
    scene->addItem(labelbins[4]);

    left_x += 60 + horiz_gap;
    labelbins[6] = new box_text(125, y_header_offset, background, background, left_x, top_y, "name", true);
    scene->addItem(labelbins[6]);

    // end label bin drawing

    // color determination adapted from iPAGE GUI (goodarzi)
    QList<QColor> colormap;
    double cmax = 30;
    double cmin = -30;
    double cmid = 0;
    QFile cmapfile(":/heatmap/firepro");
    QString cmapline;
    cmapfile.open(QIODevice::ReadOnly | QIODevice::Text);
    cmapline = cmapfile.readLine();

    while (cmapline.length()>1){
        cmapline = cmapline.replace(QRegExp("\\s$"), QString::null);
        QStringList cmaplist = cmapline.split(" ");
        colormap.append(QColor(cmaplist[0].toFloat()*255, cmaplist[1].toFloat()*255, cmaplist[2].toFloat()*255));
        cmapline = cmapfile.readLine() ;
    }

    int color_count = colormap.length()/2 ;
    // end color determination adapted from iPAGE GUI (goodarzi)

    // motif level drawing
    QGraphicsItem ***graphicbins = new QGraphicsItem **[motifcount];
    QGraphicsItem **candidatelogos = new QGraphicsItem *[motifcount];
    QGraphicsItem **candidatemi = new QGraphicsItem *[motifcount];
    QGraphicsItem **candidatezscore = new QGraphicsItem *[motifcount];
    QGraphicsItem **candidaterobust = new QGraphicsItem *[motifcount];
    QGraphicsItem **candidatenames = new QGraphicsItem *[motifcount];

    int curcluster = 0;

    for (int i = 0; i < motifcount; i++) {
        graphicbins[i] = new QGraphicsItem *[bincount];
        if (ordered_candidates[i].cluster > curcluster) {
            curcluster = ordered_candidates[i].cluster;
        }
        for (int j = 0; j < bincount; j++) {
            left_x = left_margin + x_offset + x_labelwidth + j*bin_width + j;
            top_y = top_margin + y_colorbar_offset + y_offset + y_header_offset + i*bin_height + i + curcluster*cluster_gap;

            QColor binfill = QColor(0,0,0);

            // adapted from iPAGE GUI (goodarzi)
            double v = ordered_candidates[i].bins[j].value;
            if (v > 0){
                int r = (v-cmid)/(cmax-cmid)*color_count + color_count ;
                if (r>=colormap.length()) {
                    r = colormap.length()-1 ;
                }
                binfill = colormap[r];
            }else{
                int r = (1-(v-cmid)/(cmin-cmid))*color_count ;
                if (r<0) {
                    r=0 ;
                }
                binfill = colormap[r];
            }
            // end adapted from iPAGE GUI

            QColor binborder = binfill;
            if (QString::fromStdString(ordered_candidates[i].bins[j].state) == "under") {
                binborder = QColor(0,0,255);
            } else if (QString::fromStdString(ordered_candidates[i].bins[j].state) == "over") {
                binborder = QColor(255,0,0);
            }
            graphicbins[i][j] = new box_bin(bin_width, bin_height, binfill, binborder, left_x, top_y, ordered_candidates[i].bins[j]);
            QString tooltip;
            if (QString::fromStdString(ordered_candidates[i].bins[j].state) == "over") {
                tooltip += "over-represented: ";
            } else if (QString::fromStdString(ordered_candidates[i].bins[j].state) == "under") {
                tooltip += "under-represented: ";
            }
            tooltip += "p = ";
            tooltip += QString::number(ordered_candidates[i].bins[j].p_value);
            graphicbins[i][j]->setToolTip(tooltip);
            scene->addItem(graphicbins[i][j]);
        }

        // draw motif
        left_x = left_margin + x_offset + x_labelwidth + bincount*bin_width + bincount + horiz_gap;
        top_y = top_margin + y_colorbar_offset + y_offset + y_header_offset +  + i*bin_height + i + curcluster*cluster_gap;
        QString candidate = QString::fromStdString(ordered_candidates[i].candidate);
        candidatelogos[i] = new box_text(150, bin_height, background, background, left_x, top_y, candidate,false, true, false, true);
        //candidatelogos[i] = new seqlogo(80, bin_height, left_x, top_y, candidate, background);
        candidatelogos[i]->setToolTip(candidate);
        scene->addItem(candidatelogos[i]);
        //end draw motif

        left_x = left_margin + x_offset + x_labelwidth + bincount*bin_width + bincount + horiz_gap + 120 + horiz_gap;
        top_y = top_margin + y_colorbar_offset + y_offset + y_header_offset +  + i*bin_height + i + curcluster*cluster_gap;
        candidatemi[i] = new box_text(60, bin_height, background, background, left_x, top_y, QString::number(ordered_candidates[i].mi));
        scene->addItem(candidatemi[i]);

        left_x = left_margin + x_offset + x_labelwidth + bincount*bin_width + bincount + horiz_gap + 120 + horiz_gap + 60 + horiz_gap;
        top_y = top_margin + y_colorbar_offset + y_offset + y_header_offset +  + i*bin_height + i + curcluster*cluster_gap;

        candidatezscore[i] = new box_text(60, bin_height, background, background, left_x, top_y, QString::number(ordered_candidates[i].z_score));
        scene->addItem(candidatezscore[i]);

        left_x = left_margin + x_offset + x_labelwidth + bincount*bin_width + bincount + horiz_gap + 120 + horiz_gap + 60 + horiz_gap + 60 + horiz_gap;
        top_y = top_margin + y_colorbar_offset + y_offset + y_header_offset +  + i*bin_height + i + curcluster*cluster_gap;
        candidaterobust[i] = new box_text(60, bin_height, background, background, left_x, top_y, QString::number(ordered_candidates[i].robustness) + " / " + QString::number(ordered_candidates[i].robustness_max));
        scene->addItem(candidaterobust[i]);

        left_x = left_margin + x_offset + x_labelwidth + bincount*bin_width + bincount + horiz_gap + 120 + horiz_gap + 60 + horiz_gap+ 60 + horiz_gap + 60 + horiz_gap;
        top_y = top_margin + y_colorbar_offset + y_offset + y_header_offset +  + i*bin_height + i + curcluster*cluster_gap;
        QString motifnames = "-";
        QString fullmotifnames = "-";

        for (int k = 0; k < ordered_candidates[i].names.size(); k++) {
            QString curmotifname = QString::fromStdString(ordered_candidates[i].names[k]);
            QString curmotifdetails = QString::fromStdString(ordered_candidates[i].details[k]);
            if (k == 0) {
                motifnames = curmotifname;
                fullmotifnames = curmotifname + ": " + curmotifdetails;
            }
        }
        if (motifnames.length() > 20) {
            motifnames.truncate(20);
            motifnames+= "...";
        }
        candidatenames[i] = new box_text(225, bin_height, background, background, left_x, top_y, motifnames,false, false, false, true);
        candidatenames[i]->setToolTip(fullmotifnames);
        scene->addItem(candidatenames[i]);

    }
    // end motif level drawing

}
