#include "ipage_viewer.h"
#include "box.h"
#include "header_cont.h"
#include "header_disc.h"
#include "seqlogo.h"
#include "box_text.h"
#include "box_bin.h"
#include "parser.h"
#include "scalebar.h"

#include <QtGui>
#include <QGraphicsSvgItem>
#include <QDebug>

#include <iostream>
#include <string>
#include <vector>
#include <sstream>
#include <algorithm>
#include <iterator>
#include <math.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <fstream>
#include <map>
#include <algorithm>

using namespace std;

ipage_viewer::ipage_viewer(string filename)
{
    ipage_general results = parse_ipage(filename);

    if (results.status == 1 && results.pathways.size() > 0) {
    } else {
        exit(1);
    }

    this->setWindowTitle(tr("iPAGE: ") + QString::fromStdString(filename));
    this->setWindowIcon(QIcon(":/icon/icon"));

    int pathwaycount = results.pathways.size();
    int bincount = results.bincount;
    string exptype = results.exptype;

    vector<pathway> ordered_pathways = results.pathways;

    // set up stage for drawing
    this->setViewportUpdateMode(FullViewportUpdate);
    this->setRenderHints(QPainter::Antialiasing|QPainter::TextAntialiasing);

    // set up pixel values
    int top_margin = 8;
    int bottom_margin = 8;
    int left_margin = 8;
    int right_margin = 8;

    int y_colorbar_offset = 30;
    int y_offset = 0;
    int y_header_offset = 100;
    int bin_height = 50;

    int x_offset = 0;
    int x_labelwidth = 25;

    int bin_width = 8;
    int cluster_gap = 4;
    int horiz_gap = 12;

    int scalebar_width = 550;

    // determine if space should be allocated for continuous labels
    if (QString::fromStdString(exptype) == "discrete") {
        x_labelwidth = 0;
    }

    // determine scene height, width
    int height = top_margin + y_colorbar_offset + y_offset + y_header_offset + pathwaycount*bin_height + pathwaycount + bottom_margin;
    int width = left_margin + x_offset + x_labelwidth + bincount * bin_width + bincount + horiz_gap+ 200 + right_margin;


    // increase the width of bins to suit display size
    QRect screenrect = QApplication::desktop()->screenGeometry();
    if (width < screenrect.width()) {
        int difference = (screenrect.width()-width)/bincount;
        if (difference > 16) {
            bin_width += 16;
            width += bincount * 16;
        } else {
            bin_width += difference;
            width += bincount * difference;
        }
    }

    if (width < left_margin + scalebar_width + right_margin) {
        width = left_margin + scalebar_width + right_margin;
    }

    // define scene
    QRectF bounds (0, 0, width, height);
    scene = new QGraphicsScene(bounds, this);

    // define background color; currently light grey
    QColor background = QColor(230, 230, 230);

    // add background
    box *base = new box (width, height, background, background, 0, 0, false);
    scene->addItem(base);

    scene->setItemIndexMethod(QGraphicsScene::NoIndex);
    this->setScene(scene);

    // scalebar section
    QGraphicsItem *scale = new scalebar(scalebar_width, y_colorbar_offset, left_margin, top_margin,-5,0,5,":/heatmap/ipage","under-represented", "over-represented");
    scene->addItem(scale);
    // end scalebar section

    int left_x;
    int top_y;

    // metabin drawing
    QGraphicsItem **headerbins = new QGraphicsItem *[bincount];

    if (exptype == "continuous") {
        double metabinmin = results.metabins[0].min;
        double metabinmax = results.metabins[bincount-1].max;

        QGraphicsItem **continuous_labelbins = new QGraphicsItem *[2];
        continuous_labelbins[0] = new box_text(x_labelwidth-3, y_header_offset/5, background, background, left_margin, top_margin+y_colorbar_offset, QString::number(metabinmax));
        continuous_labelbins[0]->setToolTip(QString::number(metabinmax));
        scene->addItem(continuous_labelbins[0]);

        continuous_labelbins[1] = new box_text(x_labelwidth-3, y_header_offset/5, background, background, left_margin, top_margin+y_colorbar_offset+y_header_offset-y_header_offset/5, QString::number(metabinmin));
        continuous_labelbins[1]->setToolTip(QString::number(metabinmin));
        scene->addItem(continuous_labelbins[1]);

        left_x = left_margin + x_offset + x_labelwidth;
        top_y = top_margin + y_colorbar_offset + y_offset;
        for (int i = 0; i < bincount; i++) {
            left_x = left_margin + x_offset + x_labelwidth+ i*bin_width+i;
            headerbins[i] = new header_cont(bin_width, y_header_offset, left_x, top_y, metabinmin, metabinmax, results.metabins[i].min, results.metabins[i].max, results.metabins[i]);
            QString range = QString::number(results.metabins[i].min)+ ", " + QString::number(results.metabins[i].max);
            headerbins[i]->setToolTip(range);
            scene->addItem(headerbins[i]);
        }
    } else {
        left_x = left_margin + x_offset + x_labelwidth;
        top_y = top_margin + y_colorbar_offset + y_offset;
        for (int i = 0; i < bincount; i++) {
            left_x = left_margin + x_offset + x_labelwidth+ i*bin_width+i;
            QString metabintext = QString::number(results.metabins[i].bin);
            // need to include all headers
            headerbins[i] = new header_disc(bin_width, y_header_offset, left_x, top_y, metabintext, background, results.metabins[i]);
            headerbins[i]->setToolTip(metabintext);
            scene->addItem(headerbins[i]);
        }
    }
    // end metabin drawing

    // label bin drawing
    left_x = left_margin + x_offset + x_labelwidth + bincount*bin_width+bincount + horiz_gap;
    top_y = top_margin + y_colorbar_offset + y_offset;

    QGraphicsItem **labelbins = new QGraphicsItem *[1];

    labelbins[0] = new box_text(200, y_header_offset*0.5, background, background, left_x, top_y+0.25*y_header_offset, "pathway",false, false, false, true);
    scene->addItem(labelbins[0]);
    // end label bin drawing


    // color determination adapted from iPAGE GUI (goodarzi)
    QList<QColor> colormap;
    double cmax = 5;
    double cmin = -5;
    double cmid = 0;
    QFile cmapfile(":/heatmap/ipage");
    cmapfile.open(QIODevice::ReadOnly | QIODevice::Text);
    QString cmapline = cmapfile.readLine() ;
    while (cmapline.length()>1){
        cmapline = cmapline.replace(QRegExp("\\s$"), QString::null);
        QStringList cmaplist = cmapline.split(" ");
        colormap.append(QColor(cmaplist[0].toFloat()*255, cmaplist[1].toFloat()*255, cmaplist[2].toFloat()*255));
        cmapline = cmapfile.readLine() ;
    }

    int color_count = colormap.length()/2 ;
    // end color determination adapted from iPAGE GUI (goodarzi)

    // motif level drawing
    QGraphicsItem ***graphicbins = new QGraphicsItem **[pathwaycount];
    QGraphicsItem **pathwaynames = new QGraphicsItem *[pathwaycount];

    for (int i = 0; i < pathwaycount; i++) {
        graphicbins[i] = new QGraphicsItem *[bincount];
        for (int j = 0; j < bincount; j++) {
            left_x = left_margin + x_offset + x_labelwidth + j*bin_width + j;
            top_y = top_margin + y_colorbar_offset + y_offset + y_header_offset + i*bin_height + i;

            QColor binfill = QColor(0,0,0);

            // adapted from iPAGE GUI (goodarzi)
            double v = ordered_pathways[i].bins[j].value;
            if (v > 0){
                int r = (v-cmid)/(cmax-cmid)*color_count + color_count ;
                if (r>=colormap.length()) {
                    r = colormap.length()-1 ;
                }
                binfill = colormap[r];
            }else{
                int r = (1-(v-cmid)/(cmin-cmid))*color_count ;
                if (r<0) {
                    r=0 ;
                }
                binfill = colormap[r];
            }
            // end adapted from iPAGE GUI

            QColor binborder = binfill;
            if (QString::fromStdString(ordered_pathways[i].bins[j].state) == "under") {
                binborder = QColor(0,0,255);
            } else if (QString::fromStdString(ordered_pathways[i].bins[j].state) == "over") {
                binborder = QColor(255,0,0);
            }
            graphicbins[i][j] = new box_bin(bin_width, bin_height, binfill, binborder, left_x, top_y, ordered_pathways[i].bins[j]);
            QString tooltip;
            if (QString::fromStdString(ordered_pathways[i].bins[j].state) == "over") {
                tooltip += "over-represented: ";
            } else if (QString::fromStdString(ordered_pathways[i].bins[j].state) == "under") {
                tooltip += "under-represented: ";
            }
            tooltip += "p = ";
            tooltip += QString::number(ordered_pathways[i].bins[j].p_value);
            graphicbins[i][j]->setToolTip(tooltip);
            scene->addItem(graphicbins[i][j]);
        }

        left_x = left_margin + x_offset + x_labelwidth + bincount*bin_width + bincount + horiz_gap;
        top_y = top_margin + y_colorbar_offset + y_offset + y_header_offset +  + i*bin_height + i;
        QString pathwayname = QString::fromStdString(ordered_pathways[i].pathway);
        QString fullpathwayname = QString::fromStdString(ordered_pathways[i].pathway);
        if (pathwayname.length() > 80) {
           pathwayname.truncate(80);
           pathwayname += "...";
        }

        pathwaynames[i] = new box_text(200, bin_height, background, background, left_x, top_y, pathwayname,false, true, false, true);
        pathwaynames[i]->setToolTip(fullpathwayname);
        scene->addItem(pathwaynames[i]);
        // end motif level drawing
    }
}
