#include "prmg_viewer.h"
#include "box.h"
#include "box_interact.h"
#include "box_interact_prmg.h"
#include "seqlogo.h"
#include "seqlogo_vertical.h"
#include "parser.h"
#include "scalebar.h"
#include "box_text.h"

#include <QtGui>
#include <QGraphicsSvgItem>
#include <QDebug>

#include <iostream>
#include <string>
#include <vector>
#include <sstream>
#include <algorithm>
#include <iterator>
#include <math.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <fstream>
#include <map>
#include <algorithm>

using namespace std;

prmg_viewer::prmg_viewer(string filename)
{
    fire_results fi_results = parse_fire(filename, "DNA_RNA");
    ipage_general ip_results = parse_ipage(filename);
    prmg_general results = parse_prmg(fi_results, ip_results, filename);

    if (results.status == 1) {
    } else {
        exit(1);
    }

    int motifcount = results.interaction_matrix[0].size();
    int pathwaycount = results.interaction_matrix.size();

    vector< vector<double> > imatrix = results.interaction_matrix;

    this->setWindowTitle(tr("PRMG: ") + QString::fromStdString(filename));
    this->setWindowIcon(QIcon(":/icon/icon"));

    // set up stage for drawing
    this->setViewportUpdateMode(FullViewportUpdate);
    this->setRenderHints(QPainter::Antialiasing|QPainter::TextAntialiasing);

    // set up pixel values
    int top_margin = 8;
    int bottom_margin = 8;
    int left_margin = 8;
    int right_margin = 8;

    int y_colorbar_offset = 30;
    int y_offset = 0;

    int horiz_gap = 12;

    int y_header_offset = 60+horiz_gap+40+horiz_gap+40+horiz_gap;
//    int bin_height = 50;

    int x_offset = 0;
//    int x_labelwidth = 25;

//    int bin_width = 8;
//    int cluster_gap = 4;


    int scalebar_width = 550;

    int box_width = 40;

    // determine scene height, width
    int height = top_margin + y_colorbar_offset + y_offset + y_header_offset + pathwaycount *(box_width+1) + bottom_margin;
    int width = left_margin + x_offset +  motifcount *(box_width+1) + horiz_gap+ 200 + right_margin;
    if (width < left_margin + scalebar_width + right_margin) {
        width = left_margin + scalebar_width + right_margin;
    }

    // define scene
    QRectF bounds (0, 0, width, height);
    scene = new QGraphicsScene(bounds, this);

    // define background color; currently light grey
    QColor background = QColor(230, 230, 230);

    // add background
    box *base = new box (width, height, background, background, 0, 0, false);
    scene->addItem(base);

    scene->setItemIndexMethod(QGraphicsScene::NoIndex);
    this->setScene(scene);

    // scalebar section
    QGraphicsItem *scale = new scalebar(scalebar_width, y_colorbar_offset, left_margin, top_margin,-3,0,3,":/heatmap/prmg", "under-represented", "over-represented");
    scene->addItem(scale);
    // end scalebar section

    int left_x = left_margin+x_offset;
    int top_y = top_margin+y_colorbar_offset+y_offset;

    QGraphicsItem **vertical_names = new QGraphicsItem *[motifcount];
    for (int i = 0; i < motifcount; i++) {
        QString motifnames = "-";
        QString fullmotifnames = "-";
        for (int k = 0; k < results.motifs[i].names.size(); k++) {
            QString curmotifname = QString::fromStdString(results.motifs[i].names[k]);
            curmotifname.replace(QRegExp("^M[\\d]+_"), "");
            curmotifname.replace(QRegExp("^J_MA[\\d]+_"), "");
            curmotifname.replace(QRegExp("\.ace$"), "");

            if (k == 0) {
                motifnames += curmotifname;
            }

            fullmotifnames += ", " + curmotifname;
        }
        if (motifnames.length() > 1) {
            motifnames.remove(0,1);
            fullmotifnames.remove(0,2);
        }

        vertical_names[i] = new box_text(box_width, 40, background, background, left_x + i*(box_width+1), top_y, motifnames, true, false, true);

        vertical_names[i]->setToolTip(fullmotifnames);

        scene->addItem(vertical_names[i]);
    }

    QGraphicsItem **vertical_locations = new QGraphicsItem *[motifcount];
    for (int i = 0; i < motifcount; i++) {
        vertical_locations[i] = new box_text(box_width, 40, background, background, left_x + i*(box_width+1), top_y+horiz_gap+40, QString::fromStdString(results.motifs[i].location), true, false, true);

        if (QString::fromStdString(results.motifs[i].location) == "5'") {
            vertical_locations[i]->setToolTip("DNA motif candidate");
        } else {
            vertical_locations[i]->setToolTip("RNA motif candidate");
        }

        scene->addItem(vertical_locations[i]);
    }

    QGraphicsItem **vertical_candidates = new QGraphicsItem *[motifcount];
    for (int i = 0; i < motifcount; i++) {
        QString candidate = QString::fromStdString(results.motifs[i].candidate);
        if (QString::fromStdString(results.motifs[i].location) == "5'") {
        } else {
            candidate = candidate.replace("T", "U");
        }
        vertical_candidates[i] = new seqlogo_vertical(box_width, 60, left_x + i*(box_width+1), top_y+horiz_gap+40+horiz_gap+40, candidate, background);
        vertical_candidates[i]->setToolTip(candidate);
        scene->addItem(vertical_candidates[i]);
    }


    // color determination adapted from iPAGE GUI (goodarzi)
    QList<QColor> colormap;
    double cmax = 3;
    double cmin = -3;
    double cmid = 0;
    QFile cmapfile(":/heatmap/prmg");
    cmapfile.open(QIODevice::ReadOnly | QIODevice::Text);
    QString cmapline = cmapfile.readLine() ;
    while (cmapline.length()>1){
        cmapline = cmapline.replace(QRegExp("\\s$"), QString::null);
        QStringList cmaplist = cmapline.split(" ");
        colormap.append(QColor(cmaplist[0].toFloat()*255, cmaplist[1].toFloat()*255, cmaplist[2].toFloat()*255));
        cmapline = cmapfile.readLine() ;
    }

    int color_count = colormap.length()/2 ;
    // end color determination adapted from iPAGE GUI (goodarzi)

    QGraphicsItem ***graphicbins = new QGraphicsItem **[pathwaycount];
    for (int i = 0; i < pathwaycount; i++) {
        graphicbins[i] = new QGraphicsItem *[motifcount];
        for (int j = 0; j < motifcount; j++) {
            int matrix_i = i;
            int matrix_j = j;

            double v = 0;

            QString mousetip = "";

            v = results.interaction_matrix[matrix_i][matrix_j];

            QColor binfill;
            // adapted from iPAGE GUI (goodarzi)
            if (v > 0){
                int r = (v-cmid)/(cmax-cmid)*color_count + color_count ;
                if (r>=colormap.length()) {
                    r = colormap.length()-1 ;
                }
                binfill = colormap[r];
            }else{
                int r = (1-(v-cmid)/(cmin-cmid))*color_count ;
                if (r<0) {
                    r=0 ;
                }
                binfill = colormap[r];
            }
            // end adapted from iPAGE GUI

            QString state = "";

            if (fabs(v) > 0) {
                if (v > 0) {
                    // over
                    mousetip += "over-represented: ";
                    state += "over";
                    v = -v;
                } else {
                    // under
                    mousetip += "under-represented: ";
                    state += "under";
                }
                mousetip += "p = "+QString::number(pow(10,v));
            } else {
                // not signif
                mousetip += "not significant";
                state += "neither";
            }

            bool plus = false;
            QColor lborder = binfill;
            QColor rborder = binfill;
            QColor tborder = binfill;
            QColor bborder = binfill;

            if (i == 0) {
               tborder = QColor(0,0,0);
            }

            if (i == pathwaycount -1) {
                bborder = QColor(0,0,0);
            }

            if (j == 0) {
                lborder = QColor(0,0,0);
            }

            if (j == motifcount -1) {
                rborder = QColor(0,0,0);
            }

            double pvalue = pow(10,v);
            vector<string> genelist;
            map<string, int> genetest;
            for(int p = 0; p < results.motifs[j].bins.size(); p++) {
                for (int q = 0; q < results.motifs[j].bins[p].genes.size(); q++) {
                    if (genetest.count(results.motifs[j].bins[p].genes[q]) == 0) {
                        genetest.insert(make_pair(results.motifs[j].bins[p].genes[q], 1));
                    }
                }
            }

            for (int p = 0; p < results.pathways[i].bins.size(); p++) {
                for (int q = 0; q < results.pathways[i].bins[p].genes.size(); q++) {
                    if (genetest.count(results.pathways[i].bins[p].genes[q]) > 0) {
                        genelist.push_back(results.pathways[i].bins[p].genes[q]);
                    }
                }
            }

            graphicbins[i][j] = new box_interact_prmg(box_width, box_width, binfill, binfill, left_x + j*(box_width+1), top_y+y_header_offset+i*(box_width+1),lborder, rborder, tborder, bborder, plus, true, state, pvalue, genelist);
            graphicbins[i][j]->setToolTip(mousetip);
            scene->addItem(graphicbins[i][j]);
        }
    }

    QGraphicsItem **pathwaynames = new QGraphicsItem *[pathwaycount];
    for (int i = 0; i < pathwaycount; i++) {
        QString pathwayname = QString::fromStdString(results.pathways[i].pathway);
        QString fullpathwayname = QString::fromStdString(results.pathways[i].pathway);
        if (pathwayname.length() > 80) {
           pathwayname.truncate(80);
           pathwayname += "...";
        }

        pathwaynames[i] = new box_text(200, box_width, background, background, left_x + motifcount*(box_width+1) + horiz_gap, top_y+y_header_offset+i*(box_width+1), pathwayname, false, true, false, true);
        pathwaynames[i]->setToolTip(fullpathwayname);
        scene->addItem(pathwaynames[i]);
    }


}
